Class: Coradoc::Markdown::Abbreviation
- Defined in:
- lib/coradoc/markdown/model/abbreviation.rb
Overview
Abbreviation model representing an abbreviation definition.
Kramdown syntax:
*[ABC]: A Big Corporation
When the abbreviation appears in the text, it will be wrapped with an tag with the definition as the title.