Class: Coradoc::Markdown::DefinitionTerm

Inherits:
Base
  • Object
show all
Defined in:
lib/coradoc/markdown/model/definition_term.rb

Overview

DefinitionTerm model representing a term in a definition list.

A term can have multiple definitions and can span multiple lines. Terms can also have IAL attributes attached.

Examples:

Simple term

term = Coradoc::Markdown::DefinitionTerm.new(text: "kramdown")

Method Summary

Methods inherited from Base

#serialize_content, #to_h, #to_md, visit, #visit